# 🧠 Python Data Structures Package

Welcome to the **Data Structures Package** — a clean, modular, and beginner-friendly Python library implementing essential data structures. This package is designed for learning, teaching, and experimenting with core data structure concepts in Python.

---

## 🚀 Features

This package includes Python implementations of the following data structures:

- 🧱 **Array**
- 🔗 **Singly Linked List**
- 🔁 **Circular Linked List**
- 🔄 **Doubly Linked List**
- 🗃️ **Stack (LIFO)**
- 📬 **Queue (FIFO)**
- 🌳 **Tree**
- 🕸️ **Graph**

Each data structure is implemented from scratch with easy-to-read code and useful methods to manipulate and visualize data.

---

## 📦 Installation

Clone this repository and use it locally:

```bash
git clone https://github.com/your-username/data_structures.git
cd data_structures
```

No external dependencies required. Just use Python 3.7+.

---

## 🧪 Usage

Each data structure is placed in its own module inside the package. Example usage:

```python
from danielle.stack import Stack

stack = Stack()
stack.push(10)
stack.push(20)
stack.pop()
```

---

## 📁 Project Structure

```
data_structures/
└── danielle/
    ├── __init__.py
    ├── array.py
    ├── linked_list.py
    ├── circular_linked_list.py
    ├── doubly_linked_list.py
    ├── stack.py
    ├── queue.py
    ├── tree.py
    ├── graph.py
    └── __main__.py
```

- Each `.py` file represents one data structure
- `__main__.py` can be used to test the structures via `python -m danielle`

---

## 🧰 Contributing

Want to improve a data structure or add new features (like visualization or performance benchmarking)? Contributions are welcome! Feel free to open issues or pull requests.

---

## 👩🏽‍💻 Author

Created with ❤️ by **Kapsa Danielle**  
African Master’s in Machine Intelligence (AMMI) 2025

---

## 📜 License

This project is open source and available under the [MIT License](LICENSE).

---

## ⭐️ Show Your Support

If you found this helpful or inspiring, give the project a ⭐️ on GitHub!
